#include <stdio.h>
#include <time.h>

#include "nr3.h"
#include "ran.h"
#include "gamma.h"
#include "deviates.h"

#include "poisson_uni.h"
#include "poisson_exp.h"

#define SEC(clk) (1.0 * clk / CLOCKS_PER_SEC)

int main(int argc, char* argv[]) {
    clock_t time_uni, time_exp, time_rou;
    double mean_uni, mean_exp, mean_rou;

    for (int lambda = 1; lambda <= 64; lambda *= 2) {
        int n = 1000 * 1000;
        Ran ran(time(0));
        Poissondev po(lambda, time(0));

        time_uni = clock();
        for (int i = 0; i < n; i++) {
            mean_uni += poisson_uni(&ran, lambda);    
        }
        mean_uni /= n;
        time_uni = clock() - time_uni;

        time_exp = clock();
        for (int i = 0; i < n; i++) {
            mean_exp += poisson_exp(&ran, lambda);    
        }
        mean_exp /= n;
        time_exp = clock() - time_exp;

        time_rou = clock();
        for (int i = 0; i < n; i++) {
            mean_rou += po.dev();    
        }
        mean_rou /= n;
        time_rou = clock() - time_rou;

        printf("Numbers generated per algorithm: %d\n", n);
        printf("Expected value: %d\n", lambda);
        
        printf("%10s %10s %10s\n", "method", "mean", "time [s]");
        printf("%10s %10.3f %10.3f\n", "uni", mean_uni, SEC(time_uni));
        printf("%10s %10.3f %10.3f\n", "exp", mean_exp, SEC(time_exp));
        printf("%10s %10.3f %10.3f\n", "rou", mean_rou, SEC(time_rou));
        printf("\n\n");
    }
}
